# = Class: nginx
#
# This class installs and configures the nginx web server.
#
# == Parameters
#
# [*enable*]
#   Enable the nginx service. Defaults to true.
# [*defaults*]
#   Provide the default HTTP and HTTPS sites. Defaults to true.
# [*local_config*]
#   Allow unmanaged configuration files in /etc/nginx/conf.d. Defaults
#   to false.
# [*local_sites*]
#   Allow unmanaged sites in /etc/nginx/sites-available and
#   /etc/nginx/sites-enabled. Defaults to false.
# [*processes*]
#   Number of worker processes nginx should start. Defaults to '4'.
# [*connections*]
#	Number of connections each worker will accept. Defaults to '1024'.
#
# == References
#
# * http://nginx.org/en/docs/
# * http://wiki.nginx.org/Main
#
class nginx
(
	$enable = true,
	$defaults = true,
	$local_config = false,
	$local_sites = false,
	$processes = '4',
	$connections = '1024'
)
{
	# platform specific settings
	case $::osfamily {
        redhat: {
            $user = 'nginx'
        }
        debian: {
            $user = 'www-data'
        }
        default: {
            fail("Module ${module_name} is not supported on ${::osfamily}.")
        }
    }

	# validate parameters
	validate_bool($enable)
	validate_bool($defaults)
	validate_bool($local_config)
	validate_bool($local_sites)

	if !is_integer($processes) or ($processes < 1) {
		fail("Parameter ${module_name}::processes must be a positive integer.")
	}

	if !is_integer($connections) or ($connections < 1) {
		fail("Parameter ${module_name}::connections must be a positive integer.")
	}

	# nginx
	package {'nginx':
		ensure => installed
	}

	file {'nginx.conf':
		ensure  => file,
		path    => '/etc/nginx/nginx.conf',
		owner   => 'root',
		group   => 'root',
		mode    => '0644',
		content => template('nginx/nginx.conf.erb'),
		require => Package['nginx']
	}

	file {'nginx-conf.d':
		ensure  => directory,
		path    => '/etc/nginx/conf.d',
		purge   => !$local_config,
		recurse => true,
		require => Package['nginx']
	}

	service {'nginx':
		ensure    => $enable,
		enable    => $enable,
		require   => [Package['nginx'], File['nginx.conf']],
		subscribe => File['nginx.conf']
	}

	# nginx sites
	file {'nginx-site.d':
		ensure  => directory,
		path    => '/etc/nginx/sites-available',
		purge   => !$local_sites,
		recurse => true,
		require => Package['nginx']
	}

	file {'nginx-enable.d':
		ensure  => directory,
		path    => '/etc/nginx/sites-enabled',
		purge   => !$local_sites,
		recurse => true,
		require => Package['nginx']
	}

	# default sites
	if $defaults {
		# http
		nginx::site {'default':
			names => [$::fqdn]
		}

		# https
		nginx::site {'default-ssl':
			ssl   => true,
			port  => '443',
			names => [$::fqdn]
		}
	}
}
