# = Class: nginx::site
#
# This class configures an nginx site. This is similar in concept to an Apache
# virtual host.
#
# == Parameters
#
# [*enable*]
#   Enable the site by creating a symbolic link in /etc/nginx/sites-enabled.
#   Defaults to true.
# [*names*]
#   Site names that this configuration apply to. This is an array. Defaults to
#   an array with the resource name as the only element.
# [*host*]
#   Hostname this site should be bound to. Defaults to undefined.
# [*port*]
#   Port this site should serve requests on. Defaults to 80.
# [*root*]
#   Web content root. Defaults to '/usr/share/nginx/www'.
# [*index*]
#   Pages used as indexes by the web server. This is an array. Defaults to an
#   array containing 'index.htm' and 'index.html'.
# [*ssl*]
#   Enable the SSL engine for this site. Defaults to false.
# [*ssl_cert*]
#   The SSL certificate the web server should use. Please note, this has to be
#   a chained certificate. Defaults to the host's self signed certificate.
# [*ssl_key*]
#   The private key for the SSL certificate. Defaults to the key for the
#   host's self signed certificate.
# [*ssl_protocols*]
#   SSL protocols the server should accept. This is an array. Defaults to
#   SSLv3 and TLSv1.
# [*ssl_ciphers*]
#   SSL ciphers the server should accept. This is a colon separated string.
#   Defaults to 'ALL:!ADH:!EXPORT56:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv3:+EXP'.
#
# == References
#
# * http://askubuntu.com/questions/49196/how-do-i-create-a-self-signed-ssl-certificate
#
define nginx::site
(
	$enable = true,
	$names = [$title],
	$host = undef,
	$port = '80',
	$root = undef,
	$index = ['index.htm', 'index.html'],
	$ssl = false,
	$ssl_cert = undef,
	$ssl_key = undef,
	$ssl_protocols = ['SSLv3', 'TLSv1'],
	$ssl_ciphers = 'ALL:!ADH:!EXPORT56:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv3:+EXP'
)
{
	# platform specific settings
	case $::osfamily {
		redhat: {
			$default_root = '/usr/share/nginx/html'
		}
		debian: {
			$default_root = '/usr/share/nginx/www'
		}
		default: {
			fail("Module ${module_name} is not supported on ${::osfamily}.")
		}
	}

	# validate parameters
	validate_bool($enable)
	validate_array($names)
	validate_array($index)
	validate_bool($ssl)
	validate_array($ssl_protocols)

	if $host != undef and (!is_domain_name($host) or !is_ip_address($host)) {
		fail("Parameter ${module_name}::host must be a hostname or IP address.")
	}

	if !is_integer($port) or ($port < 0) {
		fail("Parameter ${module_name}::port must be a positive integer.")
	}

	if $ssl and !$ssl_cert {
		fail("Parameter ${module_name}::ssl_cert is required when SSL is enabled.")
	}

	if $ssl and !$ssl_key {
		fail("Parameter ${module_name}::ssl_cert_key is required when SSL is enabled.")
	}

	# determine what address and port the server will bind to
	if $host {
		$listen = "${host}:${port}"
	}
	else {
		$listen = $port
	}

	# determine the document root
	$web_root = pick($root, $default_root)

	# site config file
	concat {"nginx-${title}":
		path    => "/etc/nginx/sites-available/${title}",
		owner   => 'root',
		group   => 'root',
		mode    => '0644',
		require => File['nginx-site.d'],
		notify  => Service['nginx']
	}

	concat::fragment {"nginx-${title}-site-01":
		target => "nginx-${title}",
		order  => '01',
		source => 'puppet:///modules/nginx/01-site.conf'
	}

	concat::fragment {"nginx-${title}-site-10":
		target  => "nginx-${title}",
		order   => '10',
		content => template('nginx/10-site.conf.erb')
	}

	concat::fragment {"nginx-${title}-site-80":
		target => "nginx-${title}",
		order  => '80',
		source => 'puppet:///modules/nginx/80-site.conf'
	}

	# link to enable the site
	$link_ensure = $enable ? {
		true  => 'link',
		false => 'absent'
	}

	file {"nginx-${title}.link":
		ensure  => $link_ensure,
		path    => "/etc/nginx/sites-enabled/${title}",
		target  => "/etc/nginx/sites-available/${title}",
		require => [File['nginx-enable.d'], Concat["nginx-${title}"]],
		notify  => Service['nginx']
	}
}
